﻿using System;
using System.Collections.Generic;
using System.Text;
using ContactMIS.IDAL;
using ContactMIS.Model;
using System.Data.SqlClient;
using ContactMIS.DBUtility;
using System.Data;

namespace ContactMIS.SQLServerDAL
{
    public class Contact: IContact
    {
        // Static constants
        private const string SQL_SELECT_Contacts = "SELECT Id, Name, Phone FROM Contact";
        private const string SQL_SELECT_Contact = "SELECT Id, Name, Phone FROM Category WHERE Name = @contactName";
        private const string PARM__Contact_Name = "@contactName";
        private const string SQL_INSERT_Contact = "Declare @ERR int; INSERT INTO Contact VALUES(@contactName, @Phone); SELECT @ERR=@@ERROR;";
        private const string PARM_USER_PHONE = "@Phone";
        private const string SQL_UPDATE_Contact = "Declare @ERR int; UPDATE Contact SET Phone=@Phone WHERE Name=@contactName; SELECT @ERR=@@ERROR;";



        public IList<ContactInfo> GetContacts()
        {
            IList<ContactInfo> contacts = new List<ContactInfo>();

            //Execute a query to read the categories
            using (SqlDataReader rdr = SqlHelper.ExecuteReader(SqlHelper.ConnectionString,
                CommandType.Text, SQL_SELECT_Contacts, null))
            {
                while (rdr.Read())
                {
                    ContactInfo cat = new ContactInfo(rdr.GetInt32(0), rdr.GetString(1), rdr.GetString(2));
                    contacts.Add(cat);
                }
            }
            return contacts;
        }

        public ContactInfo GetContact(string contactName)
        {
            //Set up a return value
            ContactInfo contact = null;

            //Create a parameter
            SqlParameter parm = new SqlParameter(PARM__Contact_Name, SqlDbType.VarChar, 10);
            //Bind the parameter
            parm.Value = contactName;

            //Execute the query	
            using (SqlDataReader rdr = SqlHelper.ExecuteReader(SqlHelper.ConnectionString, CommandType.Text,
                SQL_SELECT_Contact, parm))
            {
                if (rdr.Read())

                    contact = new ContactInfo(rdr.GetInt32(0), rdr.GetString(1), rdr.GetString(2));
                else
                    contact = new ContactInfo();
            }
            return contact;
        }

        public void Insert(string newName, string newPhone)
        {
            StringBuilder strSQL = new StringBuilder();
            // Get each commands parameter arrays
            SqlParameter[] orderParms = GetInsertContactParameters();
            SqlCommand cmd = new SqlCommand();

            // Set up the parameters
            orderParms[0].Value = newName;
            orderParms[1].Value = newPhone;          

            foreach (SqlParameter parm in orderParms)
                cmd.Parameters.Add(parm);

            // Create the connection to the database
            using (SqlConnection conn = new SqlConnection(SqlHelper.ConnectionString))
            {

                // Insert the order status
                strSQL.Append(SQL_INSERT_Contact);                 

                conn.Open();
                cmd.Connection = conn;
                cmd.CommandType = CommandType.Text;
                cmd.CommandText = strSQL.Append("SELECT @ERR").ToString();

                // Read the output of the query, should return error count
                using (SqlDataReader rdr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
                {
                    // Read the returned @ERR
                    rdr.Read();
                    // If the error count is not zero throw an exception
                    if (rdr.GetInt32(0) != 0)
                        throw new ApplicationException("DATA INTEGRITY ERROR ON ORDER INSERT - ROLLBACK ISSUED");
                }
                //Clear the parameters
                cmd.Parameters.Clear();
            }
        }


        public void UpdateByName(string Name, string newPhone)
        { }


        public void Delete(string contactName) 
        { }


        private static SqlParameter[] GetInsertContactParameters()
        {
            SqlParameter[] parms = SqlHelper.GetCachedParameters(SQL_INSERT_Contact);

            if (parms == null)
            {
                parms = new SqlParameter[] {
					new SqlParameter(PARM__Contact_Name, SqlDbType.VarChar, 80),				
					new SqlParameter(PARM_USER_PHONE, SqlDbType.VarChar, 80),
                };

                SqlHelper.CacheParameters(SQL_INSERT_Contact, parms);
            }

            return parms;
        }

    }
}
